package com.himajin2001.marubatsu.board;

/**
 * ○×ゲームのボードを表すClass。
 * 
 * ゲーム盤の升目とそこにどちらの石がおいてあるかの状態を保持する。
 * 
 * @author tsubasa
 */
public class Board {

	/**
	 * 盤面の枡の状態を表す配列。
	 */
	private STONE[][] hexes = null;

	/**
	 * 盤面の幅。
	 */
	private int width = 9;

	/**
	 * 盤面の高さ。
	 */
	private int hight = 9;

	/**
	 * 盤面の枡の総数。(width * hight)
	 */
	private int hexCount = 0;

	/**
	 * 石が置かれていない枡の総数。
	 */
	private int blankHexCount = 0;

	/**
	 * コンストラクタ。
	 */
	public Board() {
		//
	}

	/**
	 * ゲーム盤を設定された大きさで初期化する。
	 * 
	 * @throws BoardException
	 *             width,hightの設定が不正なとき。
	 */
	public void initBoard() throws BoardException {

		// パラメータチェック
		if (this.width < 3) {
			throw new BoardException("盤面の幅は3枡以上でないといけません。 width=" + this.width);
		}

		if (this.hight < 3) {
			throw new BoardException("盤面の高さは3枡以上でないといけません。 hight=" + this.hight);
		}

		// 盤面の大きさを規定の大きさに
		this.hexes = new STONE[this.hight][this.width];

		// 盤面上の石をすべて取り除く
		for (int y = 0; y < this.hight; ++y) {
			for (int x = 0; x < this.width; ++x) {
				this.hexes[y][x] = STONE.BLANK;
			}
		}

		// 枡の総数を計算
		this.hexCount = this.width * this.hight;

		// 石が置かれていない枡の総数をリセット。
		this.blankHexCount = this.hexCount;
	}

	/**
	 * 現在の盤面の状態を取得する。
	 * 
	 * @return 現在の盤面の状態。(変更不可。)
	 */
	public STONE[][] getHexes() {
		return this.hexes.clone();
	}

	/**
	 * 空いている枡の個数を取得する。
	 * 
	 * @return 空いている枡の個数。
	 */
	public int getBlankHexCount() {
		return this.blankHexCount;
	}

	/**
	 * 盤面の幅を取得する。
	 * 
	 * @return 盤面の幅。
	 */
	public int getWidth() {
		return this.width;
	}

	/**
	 * 盤面の幅を設定する。
	 * 
	 * 変更結果は{@link Board#initBoard()}を実行するまで反映されない。
	 * 
	 * @param width
	 *            盤面の新しい幅。
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * 盤面の高さを取得する。
	 * 
	 * @return 盤面の高さ。
	 */
	public int getHight() {
		return this.hight;
	}

	/**
	 * 盤面の高さを設定する。
	 * 
	 * 変更結果は{@link Board#initBoard()}を実行するまで反映されない。
	 * 
	 * @param hight
	 *            盤面の新しい高さ。
	 */
	public void setHight(int hight) {
		this.hight = hight;
	}

	/**
	 * 盤面に石を置く。
	 * 
	 * @param y
	 *            Y座標(0始点)
	 * @param x
	 *            X座標(0始点)
	 * @param stone
	 *            置く石の色
	 * @exception BoardException
	 *                既にその枡に石が置いてある場合。座標が正しくない場合。
	 */
	public void putStone(int y, int x, STONE stone) throws BoardException {
		// パラメータチェック
		if ((y < 0) || (y >= this.hight)) {
			throw new BoardException("Y座標が不正です。 y=" + y);
		}

		if ((x < 0) || (x >= this.width)) {
			throw new BoardException("X座標が不正です。 x=" + x);
		}

		if (this.blankHexCount == 0) {
			throw new BoardException("既に盤面が一杯です。");
		}

		// 置こうとしている場所に石が既に置いてないか？
		if (!this.hexes[y][x].equals(STONE.BLANK)) {
			throw new BoardException("既に石が置かれています。 y=" + y + " x=" + x
					+ " STONE=" + stone);
		}

		// 石を置く
		this.hexes[y][x] = stone;

		// 石が置かれていない枡の総数を減らす
		this.blankHexCount = this.blankHexCount - 1;
	}

	/**
	 * (非 Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		StringBuffer lin = new StringBuffer();

		// ヘッダーを表示
		buf.append("  |");

		for (int x = 0; x < this.width; x++) {
			lin.append("----"); // 横罫線
			buf.append(String.format("%2d", x)); // X座標値
			buf.append("｜");
		}
		lin.append("\n");
		buf.append("\n");
		buf.append(lin);

		// 全体
		for (int y = 0; y < this.hight; ++y) {
			// Y座標値ヘッダ
			buf.append(String.format("%2d｜", y));

			for (int x = 0; x < this.width; ++x) {
				switch (this.hexes[y][x].ordinal()) {
				case 0:
					// 空欄
					buf.append("  ｜");
					break;
				case 1:
					// 黒石
					buf.append("●｜");
					break;
				case 2:
					// 白石
					buf.append("○｜");
					break;
				default:
					buf.append("??｜");
				}
			}
			// 最後に改行+罫線
			buf.append("\n");
			buf.append(lin);
		}

		return buf.toString();
	}
}
