package com.himajin2001.marubatsu.control;

import com.himajin2001.marubatsu.board.Board;
import com.himajin2001.marubatsu.board.BoardException;
import com.himajin2001.marubatsu.board.STONE;
import com.himajin2001.marubatsu.player.IPlayer;

/**
 * ○×ゲーム全体の流れを実行するコントローラー。
 * 
 * @author tsubasa
 */
public class Game {

	/**
	 * 盤面Object。
	 */
	private Board board = null;

	/**
	 * 対戦者1(先攻)
	 */
	private IPlayer player1 = null;

	/**
	 * 対戦者2(後攻)
	 */
	private IPlayer player2 = null;

	/**
	 * 対戦者1が考えるのに使った時間。
	 */
	private long player1time = 0;

	/**
	 * 対戦者2が考えるのに使った時間。
	 */
	private long player2time = 0;

	/**
	 * そろっているかの判定をする為のロジック。
	 */
	private Judge judge = null;

	/**
	 * 決着がついたターン数
	 */
	private int turn = 0;

	/**
	 * 対戦者1の考えた回数。
	 */
	private int player1turn = 0;

	/**
	 * 対戦者2の考えた回数。
	 */
	private int player2turn = 0;

	/**
	 * 対戦を行う。
	 * 
	 * 事前に盤面、プレーヤー1,2が準備されていること。
	 * 
	 * @return 勝ったプレイヤーの番号。(0の場合は引き分け。)
	 */
	public int doGame() {
		// チェッカーにボードを設定する。
		this.judge.setBoard(this.board);
		this.turn = 1;
		this.player1time = 0;
		this.player2time = 0;
		this.player1turn = 0;
		this.player2turn = 0;

		System.out.println("\n\n====【対戦開始！！】====\n");

		// 対戦
		while (this.board.getBlankHexCount() > 0) {
			// *** 先攻のターン ***
			System.out.println("\n------ ターン:" + this.turn + "------\n");

			System.out.println("先攻(黒)：" + this.player1.getName() + "のターン");

			// 石を置く
			Position pos = new Position(-1, -1);
			long before = System.nanoTime();
			try {
				pos = this.player1.think(this.board.getHexes());
				this.player1time = this.player1time
						+ (System.nanoTime() - before);
				this.board.putStone(pos.getY(), pos.getX(), STONE.BLACK);
				this.player1turn = this.player1turn + 1;
			} catch (BoardException e) {
				// 石を置く際のExceptionなので反則負け！
				System.out
						.println("【反則】置けない座標" + pos.toString() + "に置こうとしました！");
				return 2;
			} catch (Exception e) {
				// Exceptionが出たら即時敗北。
				// 先攻の負け！！
				System.out.println("【エラー】考えている最中に火を噴きました！");
				return 2; // 後攻の勝ち
			}
			System.out.println("　　　座標" + pos.toString() + "に置きました。");
			System.out.println(this.board.toString());

			// そろっていないかをチェック
			if (this.judge.check(STONE.BLACK)) {
				// そろっていれば先攻の勝ち
				return 1;
			}

			// 盤面に空きがなければ引き分け(升目数が奇数の場合。)
			if (this.board.getBlankHexCount() == 0) {
				break;
			}

			// *** 後攻のターン ***

			System.out.println("後攻(白)：" + this.player2.getName() + "のターン");

			// 石を置く
			pos = new Position(-1, -1);
			before = System.nanoTime();
			try {
				pos = this.player2.think(this.board.getHexes());
				this.player2time = this.player2time
						+ (System.nanoTime() - before);
				this.board.putStone(pos.getY(), pos.getX(), STONE.WHITE);
				this.player2turn = this.player2turn + 1;
			} catch (BoardException e) {
				// 石を置く際のExceptionなので反則負け！
				System.out
						.println("【反則】置けない座標" + pos.toString() + "に置こうとしました！");
				return 1;
			} catch (Exception e) {
				// Exceptionが出たら即時敗北。
				// 後攻の負け！！
				System.out.println("【エラー】考えている最中に火を噴きました！");
				return 1; // 先攻の勝ち
			}
			System.out.println("　　　座標" + pos.toString() + "に置きました。");
			System.out.println(this.board.toString());

			// そろっていないかをチェック
			if (this.judge.check(STONE.WHITE)) {
				// そろっていれば後攻の勝ち
				return 2;
			}

			this.turn = this.turn + 1;
		} // 繰り返し

		// ループを抜けるときは引き分け(升目が全部埋まった。)
		System.out.println("全ての枡が埋まってしまいました。");
		return 0;
	}

	/**
	 * 対戦者1の合計実行時間を取得する。
	 * 
	 * @return 対戦者1の実行時間(nano秒)
	 */
	public long getPlayer1time() {
		return this.player1time;
	}

	/**
	 * 対戦者2の合計実行時間を取得する。
	 * 
	 * @return 対戦者2の実行時間(nano秒)
	 */
	public long getPlayer2time() {
		return this.player2time;
	}

	/**
	 * ゲーム盤面を設定する。
	 * 
	 * @param board
	 *            ゲーム盤Object。
	 */
	public void setBoard(Board board) {
		this.board = board;
	}

	/**
	 * 対戦者1を設定する。
	 * 
	 * @param player1
	 *            対戦者1のインスタンス。
	 */
	public void setPlayer1(IPlayer player1) {
		this.player1 = player1;
	}

	/**
	 * 対戦者2を設定する。
	 * 
	 * @param player2
	 *            対戦者2のインスタンス。
	 */
	public void setPlayer2(IPlayer player2) {
		this.player2 = player2;
	}

	/**
	 * そろっているかを確認するチェッカーを設定する。
	 * 
	 * @param judge
	 *            チェッカーのインスタンス。
	 */
	public void setJudge(Judge judge) {
		this.judge = judge;
	}

	/**
	 * 決着がついたターン数を取得する。
	 * 
	 * @return 決着のついたターン。
	 */
	public int getTurn() {
		return this.turn;
	}

	/**
	 * 対戦者1の実行したターン数を取得する。
	 * 
	 * @return 対戦者1の実行したターン数。
	 */
	public int getPlayer1turn() {
		return this.player1turn;
	}

	/**
	 * 対戦者2の実行したターン数を取得する。
	 * 
	 * @return 対戦者2の実行したターン数。
	 */
	public int getPlayer2turn() {
		return this.player2turn;
	}
}
