package com.himajin2001.marubatsu.control;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.himajin2001.marubatsu.board.Board;
import com.himajin2001.marubatsu.board.STONE;
import com.himajin2001.marubatsu.player.IPlayer;


/**
 * ゲームをセットアップして開始できる状態にするFactory。
 * 
 * @author tsubasa
 */
public class GameSetup {

	private List<IPlayer> players = new ArrayList<IPlayer>();

	private int width = 0;

	private int hight = 0;

	/**
	 * 対戦者1をインスタンス化する。
	 * 
	 * 指定されるClassはデフォルトコンストラクタを持っていること。
	 * 
	 * @param className
	 *            対戦者1のFQN("jp.co.xxxx.yyyy.Zzzzzz")
	 * @throws Exception
	 *             インスタンス化に問題があった。
	 */
	@SuppressWarnings("unchecked")
	public void addPlayer(String className) throws Exception {
		try {
			Class<IPlayer> clazz = (Class<IPlayer>) Class.forName(className);
			this.players.add(clazz.newInstance());
		} catch (ClassNotFoundException e) {
			// Classが見つからない
			e.printStackTrace();
			throw e;
		} catch (InstantiationException e) {
			// インスタンス化できない
			e.printStackTrace();
			throw e;
		} catch (IllegalAccessException e) {
			// デフォルトコンストラクタにアクセスできない。
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * GameのInstanceを作成する。
	 * 
	 * @return セットアップされたGameのインスタンス。
	 * @throws Exception
	 *             セットアップで問題が発生した。
	 */
	public Game getGameInstance() throws Exception {
		// 対戦者が2人登録されているか？
		if (this.players.size() != 2) {
			throw new Exception("対戦者の登録が足りないか、もしくは多すぎます。");
		}

		// 盤の大きさがセットされているか？
		if ((this.width < 3) || (this.hight < 3)) {
			throw new Exception("盤面の大きさが指定されていないか、もしくは小さすぎます。");
		}

		// 先攻・後攻のシャッフル
		Collections.shuffle(this.players);

		// 盤面の作成
		Board board = new Board();
		board.setHight(this.hight);
		board.setWidth(this.width);
		board.initBoard();

		// 審判の作成
		Judge judge = new Judge();

		// ゲームの作成
		Game game = new Game();
		game.setBoard(board);
		game.setJudge(judge);

		// Player1の初期化
		IPlayer player1 = this.players.get(0);
		player1.setWidth(this.width);
		player1.setHight(this.hight);
		player1.setStone(STONE.BLACK);
		game.setPlayer1(player1);

		// Player2の初期化
		IPlayer player2 = this.players.get(1);
		player2.setWidth(this.width);
		player2.setHight(this.hight);
		player2.setStone(STONE.WHITE);
		game.setPlayer2(player2);

		return game;
	}

	/**
	 * 盤面の幅を設定する。
	 * 
	 * @param width
	 *            幅。
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * 盤面の高さを設定する。
	 * 
	 * @param hight
	 *            高さ。
	 */
	public void setHight(int hight) {
		this.hight = hight;
	}
}
